import java.util.*;
 
import javax.persistence.*;
 

/*
Great and fast solution but it doesn't work with the following JPQL queries string:
    - "SELECT a,b FROM A a LEFT OUTER JOIN a.listB b WHERE b IS NULL" 
where the output would be [A4, null] or even throw an Exception 
instead [A3, null] [A4, null]??? NULL is only for single-valued field.
    - "SELECT a,b FROM A a LEFT OUTER JOIN a.listB b WHERE b.id IS NULL"

but it works fine with an relationship comparison:
"SELECT a,b FROM A a LEFT OUTER JOIN a.listB b WHERE b.a IS NULL"

By the way, it is a workaround for issue 146 ("SELECT a FROM A a WHERE a.list IS EMPTY")

Thanks.
*/ 
public class TestQuery {
    private final static String[] QSTR = {
        "SELECT a,b FROM A a LEFT OUTER JOIN a.listB b",
        "SELECT a,b FROM A a LEFT OUTER JOIN a.listB b WHERE b    IS NULL",
        "SELECT a,b FROM A a LEFT OUTER JOIN a.listB b WHERE b.id IS NULL",
        "SELECT a,b FROM A a LEFT OUTER JOIN a.listB b WHERE b.a  IS NULL"
    };

    public static void main(String [] args) {
        String[] querystr = args.length>0? args : QSTR;
        EntityManagerFactory emf =
            Persistence.createEntityManagerFactory("objectdb:test.tmp;drop");
        EntityManager em = emf.createEntityManager();
 
        em.getTransaction().begin();
        B b = new B();
        em.persist(b);
        A a1 = new A();
        a1.add(b);
        b = new B();
        em.persist(b);
        a1.add(b);
        em.persist(a1);
        A a2 = new A();
        em.persist(a2);
        em.persist(new B());
        em.getTransaction().commit();

        for (String q : querystr) {
            System.out.println("\"" + q + "\":");
            List resultList = em.createQuery(q).getResultList();
            for (Iterator it=resultList.iterator(); it.hasNext(); ) {
                Object obj = it.next();
                if (obj instanceof Object[]) 
                    System.out.println( Arrays.toString((Object[])obj) );
                else 
                    System.out.println( obj );
            } 
        }

        em.close();
        emf.close();
    }
 
    @Entity
    static class A {
        @Id @GeneratedValue long id;

        @OneToOne(mappedBy="a") B b;
 
        @ManyToMany(mappedBy="listA", fetch=FetchType.EAGER)
        ArrayList<B> listB = new ArrayList<B>();
 
        void add(B b) {
            listB.add(b);
            b.listA.add( this );
            this.b = b;
            b.a = this;
        } 
 
        @Override
        public String toString() {
            return "A" + id;
        }
    }
 
    @Entity
    static class B {
        @Id @GeneratedValue long id;

        @OneToOne A a;
 
        @ManyToMany(fetch=FetchType.EAGER)
        ArrayList<A> listA = new ArrayList<A>();       

        @Override
        public String toString() {
            return "B" + id;
        }
    }
}
